LCDCTRL	   .EQU 60h
LCDDATA    .EQU 40h
REGISTER   .EQU 0
KEYREAD    .EQU 80h

CTRLC   .EQU    03H             ; Control "C"
CTRLG   .EQU    07H             ; Control "G"
BKSP    .EQU    08H             ; Back space
LF      .EQU    0AH             ; Line feed
CS      .EQU    0CH             ; Clear screen
CR      .EQU    0DH             ; Carriage return
CTRLO   .EQU    0FH             ; Control "O"
CTRLQ	.EQU	11H		        ; Control "Q"
CTRLR   .EQU    12H             ; Control "R"
CTRLS   .EQU    13H             ; Control "S"
CTRLU   .EQU    15H             ; Control "U"
ESC     .EQU    1BH             ; Escape
DEL     .EQU    7FH             ; Delete



TEMPSTACK  .EQU     $20ED

						.ORG 			0
RST00				DI
						JP 			  INIT
						
            .ORG     0008H
RST08       JP				PRINTCHAR

            .ORG 0010H
RST10       JP READKEYINIT

            .ORG 0018H
RST18       LD				A, 0
						CP				0
						RET
						
WELCOMEMSG:
.BYTE     	"Z80 SBC",CR,LF,0

KEYMAP:
.BYTE				"abcdefgh"
.BYTE				"ijklmnop"
.BYTE				"qrstuvwx"
.BYTE				"yz123456"
.BYTE				"7890,.-",CR
.BYTE				"<+[]{}?",BKSP
.BYTE				"        " 

SHIFTKEYMAP:
.BYTE				"ABCDEFGH"
.BYTE				"IJKLMNOP"
.BYTE				"QRSTUVWX"
.BYTE				"YZ!#~$%&"
.BYTE				"/()=;._",CR
.BYTE				">*@|\\^",34,BKSP
.BYTE				"        " 

LINEADDR:
.BYTE				0h,40h,14h,54h							; Adresses for rows

INIT:       LD        HL,TEMPSTACK    ; Temp stack
						LD        SP,HL           ; Set up a temporary stack		
						XOR				E								; Empty E for key reading
						LD				A, 0						
						LD				(2000h), A			; Line counter for LCD	
						LD				(2001h), A			; Characters in line			
						LD 				A, 38h					
						OUT 			(LCDCTRL), A
						CALL			WAITLCD
						LD 				A, 0Eh
						OUT 			(LCDCTRL), A
						CALL			WAITLCD
						LD 				A, 01h
						OUT 			(LCDCTRL), A 
						CALL			WAITLCD 
						IM 				1
						LD        HL, WELCOMEMSG
						CALL			PRINT
						EI
						JP        $0200           ; Start BASIC COLD
					  HALT
						
PRINT:			LD       	A,(HL)          ; Get character
            OR       	A               ; Is it $00 ?
            RET      	Z               ; Then RETurn on terminator
						CALL			PRINTCHAR
						INC				HL
						JP				PRINT
						
PRINTCHAR:	CP				LF							; If character is linefeed
						JP				Z, NEWLINE			; Jump to newline (which then returns)
						CP				CR							; If CR
						RET				Z								; Do nothing
						PUSH 			AF							; Save character to print
						CP				BKSP						; Is it a backspace?
						JP				Z, BACKSP				; Then do that
						LD				A, (2001h)			; Load character count
						INC				A								; Add one character
						LD				(2001h), A			; And save
						CP				20							; Maximum reached?
						CALL			Z, NEWLINE			; Then next line
						POP				AF
						OUT				(LCDDATA), A		; Otherwise print char
						CALL 			WAITLCD
						RET
NEWLINE:		PUSH			AF
						PUSH			BC
						LD				A, (2000h)			; Load current number of lines
						INC				A
						CP				4								; Maximum lines reached?
						CALL			Z, NEWSCREEN		; Then clear screen
						LD				(2000h), A			; Save current line
						ADD				A, LINEADDR			; Add base of address array
						LD				B, 0						; Setup address for load line address
						LD				C, A
						LD				A, (BC)					; Load the address
						SET				7, A						; Set control bit for setting address
						OUT				(LCDCTRL), A		; And send
						CALL			WAITLCD
						LD				A, 0						
						LD				(2001h), A			; CR
						POP				BC
						POP 			AF
						RET
NEWSCREEN:	LD 				A, 01h
						OUT 			(LCDCTRL), A 
						CALL			WAITLCD 
						LD				A, 0
						RET
BACKSP:			LD				A, (2001h)			; Load character count
						CP				0								; Does the line contain a character?
						JP				Z, BACKSPEND		; Then do nothing
						DEC				A
						LD				(2001h), A			; Write count back
						LD				A, 10h					
						OUT				(LCDCTRL), A	  ; Move Cursor right
						CALL			WAITLCD
BACKSPEND:	POP				AF
						RET
						
						
						
WAITLCD:		PUSH 			AF
WAITAGAIN:	IN				A, (LCDCTRL)	
						BIT				7, A
						JP				NZ, WAITAGAIN
						POP				AF
						RET

READKEYINIT:PUSH			BC
						PUSH			DE
						PUSH			HL
						LD				E, 0						; E will be the last pressed key
READKEY:		LD				H, 2						; H is the line register, start with second
						LD				B, 0						; Count lines for later multiplication	
						LD				D, 0						; DE will be the adress for mask
						
NEXTKEY:		LD				A, H						
						CP				0								; All lines tried? 
						JP				Z, KEYOUT				; Then check if there was a key pressed
						OUT				(REGISTER), A		; Put current line to register
						IN				A, (KEYREAD)		; Input Keys
						SLA				H								; Next line
						INC				B
						CP				0								; Was key zero?
						JP				Z, NEXTKEY	 		; Then try again with next lines
						LD				D, 0						; In D will be the number of the key
LOGARITHM:	INC				D								; Add one per shift
						SRL				A								; Shift key right
						JP				NZ, LOGARITHM		; If not zero shift again
						DEC				D								; Was too much
						LD				A, 1						; Check first line for alt, shift, etc...
						OUT				(REGISTER), A
						IN				A, (KEYREAD)
						AND				1								; Check if first bit set (shift key pressed)
						JP				NZ, LOADSHIFT		; Then jump to read with shift
						LD				A, D						; Put read key into accu
						ADD				A, KEYMAP				; Add base of key map array
						JP				ADDOFFSET				; Jump to load key
LOADSHIFT:	LD				A, D
						ADD				A, SHIFTKEYMAP	; In this case add the base for shift		
ADDOFFSET:	ADD				A, 8						; Add 8 for every line
						DJNZ			ADDOFFSET				; Jump back (do while loop)
						SUB				8								; Since do while is one too much
TRANSKEY:		XOR				B								; Empty B
						LD				C, A						; A will be address in BC
						LD				A, (BC)					; Load key
						CP				E								; Same key?
						JP				Z, READKEY			; Then from beginning
						LD				E, A						; Otherwise save new key
						JP				READKEY					; And restart
KEYOUT:			LD				A, E
						LD				E, 0						; empty it
						OR				A								; Was a key read?
						JP				Z, READKEY			; If not restart
						;CALL			PRINTCHAR				; If yes print key
						POP				HL
						POP				DE
						POP				BC
						RET									
					

.end
